@@ -104,11 +104,13 @@ impl<'de> Deserializer<'de> {
 
     /// what remains to be parsed (including the
     /// character we peeked at, if any)
+    #[inline(always)]
     pub(crate) fn input(&self) -> &'de str {
         &self.src[self.pos..]
     }
 
     /// takes all remaining characters
+    #[inline(always)]
     pub(crate) fn take_all(&mut self) -> &'de str {
         let s = &self.src[self.pos..];
         self.pos = self.src.len();
@@ -116,6 +118,7 @@ impl<'de> Deserializer<'de> {
     }
 
     /// Look at the first character in the input without consuming it.
+    #[inline(always)]
     pub(crate) fn peek_char(&self) -> Result<char> {
         match self.input().chars().next() {
             Some(ch) => Ok(ch),
@@ -123,6 +126,10 @@ impl<'de> Deserializer<'de> {
         }
     }
 
+    /// read bytes_count bytes of a string.
+    /// The validity of pos + bytes_count as a valid UTF8 position must
+    /// have been checked before.
+    #[inline(always)]
     pub(crate) fn take_str(&mut self, bytes_count: usize) -> Result<&str> {
         if self.src.len() >= self.pos + bytes_count {
             let pos = self.pos;
@@ -133,23 +140,42 @@ impl<'de> Deserializer<'de> {
         }
     }
 
+    /// if the next bytes are s, then advance its length and return true
+    /// otherwise return false.
+    /// We do a comparison with a &[u8] to avoid the risk of trying read
+    /// at arbitrary positions and fall between valid UTF8 positions
+    #[inline(always)]
+    pub(crate) fn try_read(&mut self, s: &[u8]) -> bool {
+        if self.src.len() >= self.pos + s.len() {
+            if &self.src.as_bytes()[self.pos..self.pos + s.len()] == s {
+                self.pos += s.len();
+                return true;
+            }
+        }
+        false
+    }
+
     /// return the `len` first bytes of the input, without checking anything
     /// (assuming it has been done) nor consuming anything
+    #[inline(always)]
     pub(crate) fn start(&self, len: usize) -> &'de str {
         &self.src[self.pos..self.pos + len]
     }
 
     /// remove the next character (which is assumed to be ch)
+    #[inline(always)]
     pub(crate) fn drop(&mut self, ch: char) {
         self.advance(ch.len_utf8());
     }
 
     /// advance the cursor (assuming bytes_count is consistent with chars)
+    #[inline(always)]
     pub(crate) fn advance(&mut self, bytes_count: usize) {
         self.pos += bytes_count;
     }
 
     /// Consume the first character in the input.
+    #[inline(always)]
     pub(crate) fn next_char(&mut self) -> Result<char> {
         let ch = self.peek_char()?;
         self.drop(ch);
@@ -158,11 +184,13 @@ impl<'de> Deserializer<'de> {
 
     /// tells whether the next tree bytes are `'''` which
     /// is the start or end of a multiline string literal in Hjson
+    #[inline(always)]
     pub(crate) fn is_at_triple_quote(&self, offset: usize) -> bool {
         self.src.len() >= self.pos + offset + 3
             && &self.src[offset + self.pos..offset + self.pos + 3] == "'''"
     }
 
+    #[inline(always)]
     pub(crate) fn eat_line(&mut self) -> Result<()> {
         self.accept_quoteless_value = true;
         match self.input().find('\n') {
@@ -174,6 +202,7 @@ impl<'de> Deserializer<'de> {
         }
     }
 
+    #[inline(always)]
     pub(crate) fn eat_until_star_slash(&mut self) -> Result<()> {
         match self.input().find("*/") {
             Some(len) => {
@@ -204,6 +233,7 @@ impl<'de> Deserializer<'de> {
         }
     }
 
+    #[inline(always)]
     pub(crate) fn eat_shit(&mut self) -> Result<()> {
         self.eat_shit_and(None)
     }
@@ -260,11 +290,9 @@ impl<'de> Deserializer<'de> {
 
     /// Parse the JSON identifier `true` or `false`.
     fn parse_bool(&mut self) -> Result<bool> {
-        if self.input().starts_with("true") {
-            self.advance("true".len());
+        if self.try_read(b"true") {
             Ok(true)
-        } else if self.input().starts_with("false") {
-            self.advance("false".len());
+        } else if self.try_read(b"false") {
             Ok(false)
         } else {
             self.fail(ExpectedBoolean)
@@ -317,10 +345,12 @@ impl<'de> Deserializer<'de> {
     /// Parse a string until the next unescaped quote
     fn parse_quoted_string(&mut self) -> Result<String> {
         let mut s = String::new();
-        self.advance(1); // we go past the first "
+
+        let starting_quote = self.next_char()?;
+
         loop {
             let mut c = self.next_char()?;
-            if c == '\"' {
+            if c == starting_quote {
                 break;
             } else if c == '\\' {
                 c = match self.next_char()? {
@@ -433,7 +463,7 @@ impl<'de> Deserializer<'de> {
         let v = match ch {
             ',' | ':' | '[' | ']' | '{' | '}' => self.fail(UnexpectedChar),
             '\'' if self.is_at_triple_quote(0) => self.parse_multiline_string(),
-            '"' => self.parse_quoted_string(),
+            '"' | '\'' => self.parse_quoted_string(),
             _ => (if self.accept_quoteless_value {
                 self.parse_quoteless_str()
             } else {
@@ -477,13 +507,17 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
             '[' => self.deserialize_seq(visitor),
             '{' => self.deserialize_map(visitor),
             _ => {
-                let s = self.parse_string_value()?;
-                match s.as_ref() {
-                    "null" => visitor.visit_unit(),
-                    "false" => visitor.visit_bool(false),
-                    "true" => visitor.visit_bool(true),
-                    _ => visitor.visit_string(s),
+                if self.try_read(b"null") {
+                    return visitor.visit_none();
                 }
+                if self.try_read(b"true") {
+                    return visitor.visit_bool(true);
+                }
+                if self.try_read(b"false") {
+                    return visitor.visit_bool(false);
+                }
+                let s = self.parse_string_value()?;
+                visitor.visit_string(s)
             }
         }
     }
@@ -640,8 +674,7 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
         V: Visitor<'de>,
     {
         self.eat_shit()?;
-        if self.input().starts_with("null") {
-            self.advance("null".len());
+        if self.try_read(b"null") {
             visitor.visit_none()
         } else {
             visitor.visit_some(self)
@@ -654,8 +687,7 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
         V: Visitor<'de>,
     {
         self.eat_shit()?;
-        if self.input().starts_with("null") {
-            self.advance("null".len());
+        if self.try_read(b"null") {
             visitor.visit_unit()
         } else {
             self.fail(ExpectedNull)
